/******************************************************************************
 * @file     crc.h
 * @brief    CRC module header file.
 * @version  V2.1.0
 * @date     2-Mar-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#ifndef CRC_H
#define CRC_H

#ifdef __cplusplus
extern "C" {
#endif


#include "spc1169.h"


/**
 *  @brief  CRC interrupt type definition
 */
typedef enum
{
    CRC_INT_OPERATION_DONE      = (1U << 0),        /*!< Masked CRC calculation done interrupt       */
    CRC_INT_STREAM_OVERFLOW     = (1U << 1),        /*!< Masked CRC stream overflow interrupt        */
    CRC_INT_GLOBAL              = (1U << 2),        /*!< Global CRC interrupt flag issued to CPU     */
    CRC_INT_ALL                 = (0x7U),           /*!< Used for all interrupts                     */
} CRC_IntEnum;



/**
 *  @brief  CRC mode type definition
 */
typedef enum
{
    CRC_MODE_8_AUTOSAR          = 0,        /*!< CRC-8-AUTOSAR      */
    CRC_MODE_15_CAN             = 1,        /*!< CRC-15-CAN         */
    CRC_MODE_16_CCITT           = 2,        /*!< CRC-16-CCITT       */
    CRC_MODE_16_IBM             = 3,        /*!< CRC-16-IBM         */
    CRC_MODE_16_T10_DIF         = 4,        /*!< CRC-16-T10-DIF     */
    CRC_MODE_16_DNP             = 5,        /*!< CRC-16-DNP         */
    CRC_MODE_17_CAN             = 6,        /*!< CRC-17-CAN         */
    CRC_MODE_32_IEEE802P3       = 7,        /*!< CRC-32-IEEE802_3   */
} CRC_ModeEnum;




/**
 *  @brief  CRC endian type definition
 */
typedef enum
{
    CRC_LITTLE_ENDIAN           = 0,        /*!< LSB is last stage in LFSR              */
    CRC_BIG_ENDIAN              = 1,        /*!< LSB is first stage in LFSR             */
} CRC_EndianEnum;




/**
 *  @brief  CRC input/output data operation type definition
 */
typedef enum
{
    CRC_DATA_ORIGINAL           = 0,        /*!< Use original input/output data         */
    CRC_DATA_INVERT             = 1,        /*!< Invert each bit of input/output data   */
} CRC_DataOperationEnum;




/**
 *  @brief  The data format for each write to the CRCSTRIN register
 */
typedef enum
{
    CRC_FEED_PER_WORD           = 0,        /*!< CRCSTRIN is written per word (4 bytes) */
    CRC_FEED_PER_BYTE           = 1,        /*!< CRCSTRIN is written per byte           */
} CRC_FeedFormatEnum;




/******************************************************************************
 * @brief      Enable/Disable CRC calculate
 *
 * @param[in]  CRCx :  Select the CRC module
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_Enable(CRCx)                                                       \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_EN_Msk, CRCCTL_EN_Pos, 0x1)

#define CRC_Disable(CRCx)                                                      \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_EN_Msk, CRCCTL_EN_Pos, 0x0)




/******************************************************************************
 * @brief      Set CRC mode
 *
 * @param[in]  CRCx  :  Select the CRC module
 * @param[in]  eMode :  CRC mode defined by CRC_ModeEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_SetMode(CRCx, eMode)                                               \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_MODE_Msk, CRCCTL_MODE_Pos, (eMode))




/******************************************************************************
 * @brief      Enable/Disable CRC Continuous Frame
 *
 * @param[in]  CRCx :  Select the CRC module
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_EnableContinuousFrame(CRCx)                                        \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_CONTINUOUS_Msk, CRCCTL_CONTINUOUS_Pos, 0x1)

#define CRC_DisableContinuousFrame(CRCx)                                       \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_CONTINUOUS_Msk, CRCCTL_CONTINUOUS_Pos, 0x0)




/******************************************************************************
 * @brief      Set CRC Input Format
 *
 * @param[in]  CRCx        :  Select the CRC module
 * @param[in]  eEndian     :  Endian of Input Stream
 *                            Please go to CRC_EndianEnum for more details
 * @param[in]  eBitwiseNOT :  Bitwise NOT of the input data
 *                            Please go to CRC_DataOperationEnum for more details
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_SetInputFormat(CRCx, eEndian, eBitwiseNOT)                         \
    MODIFY_REG((CRCx)->CRCCTL,                                                 \
               CRCCTL_ENDIANIN_Msk | CRCCTL_INVIN_Msk,                         \
               ((eEndian) << CRCCTL_ENDIANIN_Pos) | ((eBitwiseNOT) << CRCCTL_INVIN_Pos))




/******************************************************************************
 * @brief      Set CRC Output Format
 *
 * @param[in]  CRCx        :  Select the CRC module
 * @param[in]  eEndian     :  Endian of Output Stream
 *                            Please go to CRC_EndianEnum for more details
 * @param[in]  eBitwiseNOT :  Bitwise NOT of the output result
 *                            Please go to CRC_DataOperationEnum for more details
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_SetOutputFormat(CRCx, eEndian, eBitwiseNOT)                        \
    MODIFY_REG((CRCx)->CRCCTL, CRCCTL_ENDIANOUT_Msk | CRCCTL_INVOUT_Msk,       \
               ((eEndian) << CRCCTL_ENDIANOUT_Pos) | ((eBitwiseNOT) << CRCCTL_INVOUT_Pos))




/******************************************************************************
 * @brief      Set CRC Feed data format
 *
 * @param[in]  CRCx        :  Select the CRC module
 * @param[in]  eFeedFormat :  Data format for each write
 *                            Please go to CRC_FeedFormatEnum for more details
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_SetFeedFormat(CRCx, eFeedFormat)                                      \
    WRITE_FIELD((CRCx)->CRCCTL, CRCCTL_PERBYTE_Msk, CRCCTL_PERBYTE_Pos, (eFeedFormat))




/******************************************************************************
 * @brief      Set CRC stream data length
 *
 * @param[in]  CRCx      :  Select the CRC module
 * @param[in]  u32StrLen :  Stream data length in byte
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_SetStreamLen(CRCx,u32StrLen)            ((CRCx)->CRCSTRLEN = ((u32StrLen) - 1U))




/******************************************************************************
 * @brief      Feed data into CRC stream
 *
 * @param[in]  CRCx    :  Select the CRC module
 * @param[in]  u32Data :  Input data
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_FeedData(CRCx,u32Data)                  ((CRCx)->CRCSTRIN = (u32Data))




/******************************************************************************
 * @brief      Get CRC result
 *
 * @param[in]  CRCx :  Select the CRC module
 *
 * @return     CRC calculation result
 *
 ******************************************************************************/
#define CRC_GetResult(CRCx)                         ((CRCx)->CRCRESULT)




/******************************************************************************
 * @brief      Enable/Disable CRC interrupt
 *
 * @param[in]  CRCx   :  Select the CRC module
 * @param[in]  eEvent :  Select the CRC event
 *                       Please goto CRC_IntEnum for selection
 * @return     none
 *
 ******************************************************************************/
#define CRC_EnableInt(CRCx,eEvent)                  SET_BITS((CRCx)->CRCIE, eEvent)
#define CRC_DisableInt(CRCx,eEvent)                 CLEAR_BITS((CRCx)->CRCIE, eEvent)




/******************************************************************************
 * @brief      Clear CRC interrupt flag
 *
 * @param[in]  CRCx   :  Select the CRC module
 * @param[in]  eEvent :  Select the CRC event
 *                       Please goto CRC_IntEnum for selection
 * @return     none
 *
 ******************************************************************************/
#define CRC_ClearInt(CRCx,eEvent)                   ((CRCx)->CRCIC = (eEvent))




/******************************************************************************
 * @brief      Get CRC interrupt flag
 *
 * @param[in]  CRCx   :  Select the CRC module
 * @param[in]  eEvent :  Select the CRC event
 *                       Please goto CRC_IntEnum for selection
 * @return     0     - Interrupt is not occurred
 *             not 0 - Interrupt is occurred
 *
 ******************************************************************************/
#define CRC_GetIntFlag(CRCx,eEvent)                 ((CRCx)->CRCIF & (eEvent))




/******************************************************************************
 * @brief      Get CRC interrupt raw flag
 *
 * @param[in]  CRCx   :  Select the CRC module
 * @param[in]  eEvent :  Select the CRC event
 *                       Please goto CRC_IntEnum for selection
 * @return     0     - Interrupt is not occurred
 *             not 0 - Interrupt is occurred
 *
 ******************************************************************************/
#define CRC_GetIntRawFlag(CRCx,eEvent)              ((CRCx)->CRCRAWIF & (eEvent))




/******************************************************************************
 * @brief      Enable/Disable write access to the protected CRC registers
 *
 * @param[in]  CRCx :  Select the CRC module
 *
 * @return     none
 *
 ******************************************************************************/
#define CRC_WALLOW(CRCx)                            ((CRCx)->CRCREGKEY = 0x1ACCE551)
#define CRC_WDIS(CRCx)                              ((CRCx)->CRCREGKEY = 0x0)




/**
 *  @brief  CRC Public Function Declaration
 */
void CRC_Init(CRC_REGS *CRCx, CRC_ModeEnum eMode);
uint32_t CRC_Calculate(CRC_REGS *CRCx, const uint8_t *pu8DataStr, uint32_t u32DataLen);
void CRC_FeedStream(CRC_REGS *CRCx, const uint8_t *pu8DataStr, uint32_t u32DataLen);


#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* CRC_H */


/******************* Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd. ***** END OF FILE ****/

