/******************************************************************************
 * @file     crc.c
 * @brief    CRC firmware functions.
 * @version  V2.1.0
 * @date     2-Mar-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "crc.h"




/****************************************************************************//**
 * @brief      CRC Initial
 *
 * @param[in]  CRCx  :  Select the CRC module
 * @param[in]  eMode :  CRC mode defined by CRC_ModeEnum
 *
 * @return     none
 *
 *******************************************************************************/
void CRC_Init(CRC_REGS *CRCx, CRC_ModeEnum eMode)
{
    /* Set CRC mode */
    CRC_SetMode(CRCx, eMode);

    /* LITTLE_ENDIAN, put the original data into LFSR */
    CRC_SetInputFormat(CRCx, CRC_LITTLE_ENDIAN, DISABLE);

    /* LITTLE_ENDIAN, Output result is the original value in LFSR */
    CRC_SetOutputFormat(CRCx, CRC_LITTLE_ENDIAN, DISABLE);

    /* Internal register will be cleared when CRCCTL.EN=0 so next frame calculation is a fresh new start */
    CRC_DisableContinuousFrame(CRCx);

    /* CRCSTRIN is written per word (4 bytes) */
    CRC_SetFeedFormat(CRCx, CRC_FEED_PER_WORD);

    CRC_Enable(CRCx);
}




/****************************************************************************//**
 * @brief      Make input data stream 4 bytes aligned and feed data in CRC
 *
 * @param[in]  CRCx       :  Select the CRC module
 * @param[in]  pu8DataStr :  input data stream
 * @param[in]  u32DataLen :  data length in byte
 *
 * @return     CRC result(32-bit)
 *
 *******************************************************************************/
uint32_t CRC_Calculate(CRC_REGS *CRCx, const uint8_t *pu8DataStr, uint32_t u32DataLen)
{
    uint32_t cnt;
    uint32_t u32Divisor, u32Tail;

    /* CRCSTRIN is written per word (4 bytes) */
    CRC_SetFeedFormat(CRCx, CRC_FEED_PER_WORD);

    /* Set stream data length */
    CRC_SetStreamLen(CRCx, u32DataLen);

    /* Padding first and copy data in */
    u32Divisor = u32DataLen / 4;
    u32Tail    = u32DataLen % 4;

    switch(u32Tail)
    {
        case 1:
            CRCx->CRCSTRIN     = (((uint32_t)pu8DataStr[0]) << 24);
            break;

        case 2:
            CRCx->CRCSTRIN     = ((((uint32_t)pu8DataStr[0]) << 16) |
                                  (((uint32_t)pu8DataStr[1]) << 24));
            break;

        case 3:
            CRCx->CRCSTRIN     = ((((uint32_t)pu8DataStr[0]) << 8)  |
                                  (((uint32_t)pu8DataStr[1]) << 16) |
                                  (((uint32_t)pu8DataStr[2]) << 24));
            break;

        default:
            break;
    }

    /* Compose last data from byte to word and copy data in  */
    for(cnt = 0; cnt < u32Divisor; cnt++)
    {
        CRCx->CRCSTRIN     = (((uint32_t)pu8DataStr[4 * cnt + u32Tail    ])        |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 1]) << 8)  |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 2]) << 16) |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 3]) << 24));
    }

    /* Wait for the result */
    while(CRC_GetIntRawFlag(CRCx, CRC_INT_OPERATION_DONE) == 0)
    {}

    /* Clear interrupt flag */
    CRC_ClearInt(CRCx, CRC_INT_ALL);

    /* Return the result */
    return CRC_GetResult(CRCx);
}




/****************************************************************************//**
 * @brief      Feed data stream to CRC input register
 *
 * @param[in]  CRCx       :  Select the CRC module
 * @param[in]  pu8DataStr :  Input data stream
 * @param[in]  u32DataLen :  Data length in byte
 *
 * @return     none
 *
 *******************************************************************************/
void CRC_FeedStream(CRC_REGS *CRCx, const uint8_t *pu8DataStr, uint32_t u32DataLen)
{
    uint32_t cnt;
    uint32_t u32Divisor, u32Tail;

    /* CRCSTRIN is written per word (4 bytes) */
    CRC_SetFeedFormat(CRCx, CRC_FEED_PER_WORD);

    /* Set stream data length */
    CRC_SetStreamLen(CRCx, u32DataLen);

    /* Padding first and copy data in */
    u32Divisor = u32DataLen / 4;
    u32Tail    = u32DataLen % 4;

    switch(u32Tail)
    {
        case 1:
            CRCx->CRCSTRIN     = (((uint32_t)pu8DataStr[0]) << 24);
            break;

        case 2:
            CRCx->CRCSTRIN     = ((((uint32_t)pu8DataStr[0]) << 16) |
                                  (((uint32_t)pu8DataStr[1]) << 24));
            break;

        case 3:
            CRCx->CRCSTRIN     = ((((uint32_t)pu8DataStr[0]) << 8)  |
                                  (((uint32_t)pu8DataStr[1]) << 16) |
                                  (((uint32_t)pu8DataStr[2]) << 24));
            break;

        default:
            break;
    }

    /* Compose last data from byte to word and copy data in  */
    for(cnt = 0; cnt < u32Divisor; cnt++)
    {
        CRCx->CRCSTRIN     = (((uint32_t)pu8DataStr[4 * cnt + u32Tail    ])        |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 1]) << 8)  |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 2]) << 16) |
                              (((uint32_t)pu8DataStr[4 * cnt + u32Tail + 3]) << 24));
    }
}




/******************* Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd. ***** END OF FILE ****/

